@@ -3,7 +3,7 @@ __license__ = 'GPL 3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, re, sys, shutil, pprint
+import os, re, sys, shutil, pprint, json
 from functools import partial
 
 from calibre.customize.conversion import OptionRecommendation, DummyReporter
@@ -40,15 +40,18 @@ various stages of conversion. The stages are:
 
 '''
 
+
 def supported_input_formats():
     fmts = available_input_formats()
     for x in ('zip', 'rar', 'oebzip'):
         fmts.add(x)
     return fmts
 
+
 class OptionValues(object):
     pass
 
+
 class CompositeProgressReporter(object):
 
     def __init__(self, global_min, global_max, global_reporter):
@@ -60,8 +63,10 @@ class CompositeProgressReporter(object):
                 (self.global_max - self.global_min)
         self.global_reporter(global_frac, msg)
 
+
 ARCHIVE_FMTS = ('zip', 'rar', 'oebzip')
 
+
 class Plumber(object):
 
     '''
@@ -135,7 +140,7 @@ OptionRecommendation(name='output_profile',
             choices=[x.short_name for x in output_profiles()],
             help=_('Specify the output profile. The output profile '
                    'tells the conversion system how to optimize the '
-                   'created document for the specified device. In some cases, '
+                   'created document for the specified device (such as by resizing images for the device screen size). In some cases, '
                    'an output profile can be used to optimize the output for a particular device, but this is rarely necessary. '
                    'Choices are:') +
                            ', '.join([x.short_name for x in output_profiles()])
@@ -355,6 +360,12 @@ OptionRecommendation(name='extra_css',
                 'rules.')
         ),
 
+OptionRecommendation(name='transform_css_rules',
+            recommended_value=None, level=OptionRecommendation.LOW,
+            help=_('Rules for transforming the styles in this book. These'
+                   ' rules are applied after all other CSS processing is done.')
+        ),
+
 OptionRecommendation(name='filter_css',
             recommended_value=None, level=OptionRecommendation.LOW,
             help=_('A comma separated list of CSS properties that '
@@ -490,7 +501,7 @@ OptionRecommendation(name='smarten_punctuation',
         recommended_value=False, level=OptionRecommendation.LOW,
         help=_('Convert plain quotes, dashes and ellipsis to their '
             'typographically correct equivalents. For details, see '
-            'http://daringfireball.net/projects/smartypants'
+            'https://daringfireball.net/projects/smartypants'
             )
         ),
 
@@ -767,7 +778,7 @@ OptionRecommendation(name='search_replace',
         # plugins.
         for w in ('input_options', 'output_options',
                 'all_format_options'):
-            temp = set([])
+            temp = set()
             for x in getattr(self, w):
                 temp.add(x.clone())
             setattr(self, w, temp)
@@ -825,13 +836,16 @@ OptionRecommendation(name='search_replace',
         if help is not None:
             return help.replace('%default', str(rec.recommended_value))
 
+    def merge_plugin_recs(self, plugin):
+        for name, val, level in plugin.recommendations:
+            rec = self.get_option_by_name(name)
+            if rec is not None and rec.level <= level:
+                rec.recommended_value = val
+                rec.level = level
+
     def merge_plugin_recommendations(self):
         for source in (self.input_plugin, self.output_plugin):
-            for name, val, level in source.recommendations:
-                rec = self.get_option_by_name(name)
-                if rec is not None and rec.level <= level:
-                    rec.recommended_value = val
-                    rec.level = level
+            self.merge_plugin_recs(source)
 
     def merge_ui_recommendations(self, recommendations):
         '''
@@ -1098,7 +1112,7 @@ OptionRecommendation(name='search_replace',
         pr(0.35)
         self.flush()
 
-        if self.output_plugin.file_type != 'epub':
+        if self.output_plugin.file_type not in ('epub', 'kepub'):
             # Remove the toc reference to the html cover, if any, except for
             # epub, as the epub output plugin will do the right thing with it.
             item = getattr(self.oeb.toc, 'item_that_refers_to_cover', None)
@@ -1153,12 +1167,18 @@ OptionRecommendation(name='search_replace',
         mobi_file_type = getattr(self.opts, 'mobi_file_type', 'old')
         needs_old_markup = (self.output_plugin.file_type == 'lit' or
                     (self.output_plugin.file_type == 'mobi' and mobi_file_type == 'old'))
+        transform_css_rules = ()
+        if self.opts.transform_css_rules:
+            transform_css_rules = self.opts.transform_css_rules
+            if isinstance(transform_css_rules, basestring):
+                transform_css_rules = json.loads(transform_css_rules)
         flattener = CSSFlattener(fbase=fbase, fkey=fkey,
                 lineh=line_height,
                 untable=needs_old_markup,
                 unfloat=needs_old_markup,
                 page_break_on_body=self.output_plugin.file_type in ('mobi',
                     'lit'),
+                transform_css_rules=transform_css_rules,
                 specializer=partial(self.output_plugin.specialize_css_for_output,
                     self.log, self.opts))
         flattener(self.oeb, self.opts)
@@ -1212,13 +1232,17 @@ OptionRecommendation(name='search_replace',
         self.log(self.output_fmt.upper(), 'output written to', self.output)
         self.flush()
 
+
 # This has to be global as create_oebbook can be called from other locations
 # (for example in the html input plugin)
 regex_wizard_callback = None
+
+
 def set_regex_wizard_callback(f):
     global regex_wizard_callback
     regex_wizard_callback = f
 
+
 def create_oebbook(log, path_or_stream, opts, reader=None,
         encoding='utf-8', populate=True, for_regex_wizard=False, specialize=None):
     '''
